//
//  FHAgentInviteTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/2/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "macros.h"


@interface FHAgentInviteTests : XCTestCase
@property (nonatomic, strong) FHAgent       *agent;
@property (nonatomic, strong) FHCompany     *company;
@property (nonatomic, strong) FHAgentInvite *agentInvite;
@end


@implementation FHAgentInviteTests


+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent = next_test_agent();
    _company = [_agent.companies lastObject];

    _agentInvite = [FHAgentInvite agentInviteForCompany:_company toEmail:[MTFaker email]];
    MTPocketRequest *request = [_agentInvite save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
}

- (void)testCreateAgentInvite
{
    FHAgentInvite *agentInvite = [FHAgentInvite agentInviteForCompany:_company toEmail:[MTFaker email]];
    MTPocketRequest *request = [agentInvite save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
}

- (void)testResendAgentInvite
{
    MTPocketRequest *request = [_agentInvite resend];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testDestroyAgentInvite
{
    MTPocketRequest *request = [_agentInvite destroy];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}


@end



