//
//  FHAgentTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/12/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "macros.h"
#import "MTFaker.h"
#import "FHClient.h"


@interface FHAgentTests : XCTestCase
@property (nonatomic, strong) FHAgent *agent;
@end


@implementation FHAgentTests

+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}

- (void)setUp
{
    _agent = next_test_agent();
}

- (void)testSignUp
{
    FHAgent *agent = [FHAgent agentWithEmail:[MTFaker email] password:[MTFaker password]];
    MTPocketRequest *request = [agent signupWithFirstName:[MTFaker name] lastName:[MTFaker name]];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
    XCTAssertNotNil(_agent.firstName);
    XCTAssertNotNil(_agent.lastName);
    XCTAssertNotNil(_agent.email);
    XCTAssertNotNil(_agent.identifier);
    XCTAssertNotNil(_agent.createdAt);
    XCTAssertTrue(_agent.companies.count == 1);
}

- (void)testLogin
{
    MTPocketRequest *request = [_agent login];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertNotNil(_agent.firstName);
    XCTAssertNotNil(_agent.lastName);
    XCTAssertNotNil(_agent.email);
    XCTAssertNotNil(_agent.identifier);
    XCTAssertNotNil(_agent.createdAt);
    XCTAssertTrue(_agent.companies.count == 1);
}

- (void)testFetch
{
    MTPocketRequest *request = [_agent fetch];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertNotNil(_agent.firstName);
    XCTAssertNotNil(_agent.lastName);
    XCTAssertNotNil(_agent.email);
    XCTAssertNotNil(_agent.identifier);
    XCTAssertNotNil(_agent.createdAt);
    XCTAssertTrue(_agent.companies.count == 1);
    FHCompany *company = [_agent.companies lastObject];
    XCTAssertNotNil(company.identifier);
    XCTAssertNotNil(company.title);
    XCTAssertNotNil(company.token);
    XCTAssertTrue(company.fetchAutomatically);
    XCTAssertTrue(company.numberOfAccounts == 3);
    XCTAssertNotNil(company.createdAt);
}

- (void)testUpdate
{
    MTPocketRequest *request = nil;

    _agent.firstName    = @"Heidi";
    _agent.lastName     = @"Guest";
    [_agent setPassword:[MTFaker password]];


    request = [_agent save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);


    request = [_agent fetch];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue([_agent.firstName isEqualToString:@"Heidi"]);
    XCTAssertTrue([_agent.lastName isEqualToString:@"Guest"]);
    XCTAssertNotNil(_agent.email);
    XCTAssertNotNil(_agent.identifier);
    XCTAssertNotNil(_agent.createdAt);
    XCTAssertTrue(_agent.companies.count == 1);
}

- (void)testDestroy
{
    MTPocketRequest *request = [_agent destroy];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}


@end
