//
//  FHCannedResponseTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/2/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "FHClient.h"
#import "macros.h"


@interface FHCannedResponseTests : XCTestCase
@property (nonatomic, strong) FHAgent          *agent;
@property (nonatomic, strong) FHCompany        *company;
@property (nonatomic, strong) FHCannedResponse *cannedResponse;
@end


@implementation FHCannedResponseTests

+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent = next_test_agent();
    _company = [_agent.companies lastObject];

     _cannedResponse = [FHCannedResponse cannedResponseForCompany:_company withName:[MTFaker name] text:[MTFaker lorem:10]];
    MTPocketRequest *request = [_cannedResponse save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
}

- (void)testCreateCannedResponse
{
    FHCannedResponse *cannedResponse = [FHCannedResponse cannedResponseForCompany:_company withName:[MTFaker name] text:[MTFaker lorem:10]];
    MTPocketRequest *request = [cannedResponse save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
}

- (void)testUpdateCannedResponse
{
    _cannedResponse.shortcut = [MTFaker name];
    MTPocketRequest *request = [_cannedResponse save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testDestroyCannedResponse
{
    MTPocketRequest *request = [_cannedResponse destroy];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}


@end
