//
//  FHCompanyTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/14/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "macros.h"
#import "FHClient.h"


@interface FHCompanyTests : XCTestCase
@property (nonatomic, strong) FHAgent   *agent;
@property (nonatomic, strong) FHCompany *company;
@end


@implementation FHCompanyTests

+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent      = next_test_agent();
    _company    = [_agent.companies lastObject];
}

- (void)testCreate
{
    FHCompany *company          = [FHCompany companyWithAgent:_agent title:[MTFaker name]];
    MTPocketRequest *request    = [company save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
    XCTAssertNotNil(company.identifier);
    XCTAssertNotNil(company.title);
    XCTAssertNil(company.lastFetchAt);
    XCTAssertTrue(company.fetchAutomatically);
    XCTAssertNotNil(company.createdAt);
    XCTAssertTrue(company.agents.count == 1);
}

- (void)testFetch
{
    MTPocketRequest *request = [_company fetch];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertNotNil(_company.identifier);
    XCTAssertNotNil(_company.title);
    XCTAssertNil(_company.lastFetchAt);
    XCTAssertTrue(_company.fetchAutomatically);
    XCTAssertNotNil(_company.createdAt);
    XCTAssertTrue(_company.agents.count > 0);
    XCTAssertTrue(_company.agentInvites.count > 0);
    XCTAssertTrue(_company.cannedResponses.count > 0);
    XCTAssertTrue(_company.tags.count > 0);
}

- (void)testUpdate
{
    _company.title = [MTFaker name];
    MTPocketRequest *request = [_company save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testDestroy
{

    // add company to destroy
    FHCompany *company          = [FHCompany companyWithAgent:_agent title:[MTFaker name]];
    MTPocketRequest *request    = [company save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)

    // test destorying the users default company
    request = [_company destroy];
    resp    = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testForceChannelsFetch
{
    MTPocketRequest *request = [_company forceChannelsFetch];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testFetchNotifications
{
    FHRemoteArray *notifications = _company.notifications;
    MTPocketRequest *request = [notifications next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(notifications.count > 0);
}

- (void)testFetchTwitterAccounts
{
    FHRemoteArray *twitterAccounts = _company.twitterAccounts;
    MTPocketRequest *request = [twitterAccounts next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(twitterAccounts.count > 0);
}

- (void)testFetchFacebookAccounts
{
    FHRemoteArray *facebookAccounts = _company.facebookAccounts;
    MTPocketRequest *request = [facebookAccounts next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(facebookAccounts.count > 0);
}

- (void)testFetchEmailAccounts
{
    FHRemoteArray *emailAccounts = _company.emailAccounts;
    MTPocketRequest *request = [emailAccounts next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(emailAccounts.count > 0);
}

- (void)testAddAndRemoveAgent
{
    MTPocketRequest *request = nil;

    FHAgent *agent = next_test_agent();

    // log in the agent that owns the company
    request = [_agent login];
    __block MTPocketResponse *resp = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    // fetch company
    request = [_company fetch];
    resp = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    // get current agent count
    NSInteger count = _company.agents.count;

    // add agent
    request = [_company addAgent:agent];
    resp = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);

    // fetch company
    request = [_company fetch];
    resp = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    // check agent was added
    XCTAssertTrue(_company.agents.count == count + 1);

    // remove agent
    request = [_company removeAgent:agent];
    resp = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);

    // fetch company
    request = [_company fetch];
    resp = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    // check agent was removed
    XCTAssertTrue(_company.agents.count == count);
}


@end
