//
//  FHCustomerTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/1/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import <NSDate+MTDates.h>
#import "MTFaker.h"
#import "FHClient.h"
#import "macros.h"


@interface FHCustomerTests : XCTestCase
@property (nonatomic, strong) FHAgent   *agent;
@property (nonatomic, strong) FHCompany *company;
@end


@implementation FHCustomerTests

+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent      = next_test_agent();
    _company    = [_agent.companies lastObject];
}

- (void)testListCustomers
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];

    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);
}

- (void)testFetch
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterUnresolved
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];

    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

    FHCustomer *customer = [customers lastObject];

    request = [customer fetch];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertNotNil(customer.identifier);
    XCTAssertNotNil(customer.name);
    XCTAssertNotNil(customer.location);
    XCTAssertNotNil(customer.timeZone);
    XCTAssertNotNil(customer.createdAt);
    XCTAssertTrue(customer.customerAccounts.count > 0);
    XCTAssertTrue(customer.interactionFlaggedAgents.count > 0);
}

- (void)testDestroy
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterUnresolved
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];

    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

    FHCustomer *customer = [customers lastObject];

    request = [customer destroy];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testResolveAllInteractions
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterUnresolved
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];

    MTPocketRequest *request = [customers next];
    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);



    FHCustomer *customer = [customers lastObject];
    request = [customer resolveAllInteractions];
    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
    XCTAssertTrue(customers.count > 0);
}

- (void)testListCustomersUnresolved
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterUnresolved
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];

    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);
}

- (void)testListCustomersPaginated
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    customers.perPage   = 2;
    customers.page      = 2;

    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);
}

- (void)testListCustomersSortNewestFirst
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

    FHCustomer *first   = customers[0];
    FHCustomer *last    = [customers lastObject];
    XCTAssertTrue([first.newestInteractionReceivedAt mt_isAfter:last.newestInteractionReceivedAt]);
}

- (void)testListCustomersSortOldestFirst
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortOldestFirst];
    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

    FHCustomer *first   = customers[0];
    FHCustomer *last    = [customers lastObject];
    XCTAssertTrue([first.newestInteractionReceivedAt mt_isBefore:last.newestInteractionReceivedAt]);
}

- (void)testListCustomersFilterByEmailChannel
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterEmail
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

    for (FHCustomer *customer in customers) {
        FHCustomerAccount *customerAccount = [customer.customerAccounts lastObject];
        XCTAssertTrue(customerAccount.channel == FHInteractionChannelEmail);
    }
}

- (void)testListCustomersFilterByFacebookChannel
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterFacebook
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

    for (FHCustomer *customer in customers) {
        FHCustomerAccount *customerAccount = [customer.customerAccounts lastObject];
        XCTAssertTrue(customerAccount.channel == FHInteractionChannelFacebook);
    }
}

- (void)testListCustomersFilterByTwitterChannel
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterTwitter
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

    for (FHCustomer *customer in customers) {
        FHCustomerAccount *customerAccount = [customer.customerAccounts lastObject];
        XCTAssertTrue(customerAccount.channel == FHInteractionChannelTwitter);
    }
}

- (void)testListCustomersSearchForKeywords
{
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:@"a"   // could potentially false fail, but not likely
                                                                    sort:FHCustomerSortNewestFirst];
    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

}

- (void)testCustomerHasAgentWithDibs
{
    // grab customers
    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    MTPocketRequest *request = [customers next];
    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];
    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    FHCustomer *customer = [customers lastObject];

    // fetch customer interactions (calling dibs on the customer)
    FHRemoteArray *interactions = customer.interactions;
    request = [interactions next];
    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];
    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue([interactions count] > 0);

    // fetch customers
    customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                            channelFilter:FHCustomerChannelFilterAll
                                             searchString:nil
                                                     sort:FHCustomerSortNewestFirst];
    request = [customers next];
    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];
    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(customers.count > 0);

    // verify that the customer has a dibs agent
    BOOL anyCustomerHasAnAgentWithDibs = NO;
    for (FHCustomer *c in customers) {
        if (c.agentWithDibs) {
            anyCustomerHasAnAgentWithDibs = YES;
            break;
        }
    }
    XCTAssertTrue(anyCustomerHasAnAgentWithDibs);
}


// TODO finish writing tests for other cases like tags and agents in the search_text param



@end
