//
//  FHEmailAccountTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/2/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "FHClient.h"
#import "macros.h"


@interface FHEmailAccountTests : XCTestCase
@property (nonatomic, strong) FHAgent        *agent;
@property (nonatomic, strong) FHCompany      *company;
@property (nonatomic, strong) FHEmailAccount *emailAccount;
@end


@implementation FHEmailAccountTests


+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent = next_test_agent();
    _company = [_agent.companies lastObject];
    _emailAccount = [FHEmailAccount emailAccountForCompany:_company
                                                      kind:FHEmailAccountKindIMAP
                                                    server:@"mail.example.com"
                                                      port:537
                                                  username:[MTFaker name]
                                                  password:[MTFaker password]
                                                  usingSSL:YES];
    MTPocketRequest *request = [_emailAccount save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
}

- (void)testCreateEmailAccount
{
    _emailAccount = [FHEmailAccount emailAccountForCompany:_company
                                                      kind:FHEmailAccountKindIMAP
                                                    server:@"mail.example.com"
                                                      port:537
                                                  username:[MTFaker name]
                                                  password:[MTFaker password]
                                                  usingSSL:YES];

    MTPocketRequest *request = [_emailAccount save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
}

- (void)testUpdateEmailAccount
{
    _emailAccount.server = @"mailbox.example.com";
    MTPocketRequest *request = [_emailAccount save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testDestroyEmailAccount
{
    MTPocketRequest *request = [_emailAccount destroy];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testGuessFieldsFromEmail
{
    // gmail
    _emailAccount.username = @"test@gmail.com";
    [_emailAccount guessFieldsFromEmail];
    XCTAssertTrue(_emailAccount.kind == FHEmailAccountKindIMAP);
    XCTAssertTrue(_emailAccount.SSL  == YES);
    XCTAssertTrue(_emailAccount.port == 993);
    XCTAssertTrue([_emailAccount.server isEqualToString:@"imap.googlemail.com"]);

    // hotmail
    _emailAccount.username = @"test@hotmail.com";
    [_emailAccount guessFieldsFromEmail];
    XCTAssertTrue(_emailAccount.kind == FHEmailAccountKindPOP);
    XCTAssertTrue(_emailAccount.SSL  == YES);
    XCTAssertTrue(_emailAccount.port == 995);
    XCTAssertTrue([_emailAccount.server isEqualToString:@"pop3.live.com"]);

}


@end





