//
//  FHFacebookAccountTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/2/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "FHClient.h"
#import "macros.h"


@interface FHFacebookAccountTests : XCTestCase
@property (nonatomic, strong) FHAgent           *agent;
@property (nonatomic, strong) FHCompany         *company;
@property (nonatomic, strong) FHFacebookAccount *facebookAccount;
@end


@implementation FHFacebookAccountTests


+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent = next_test_agent();
    _company = [_agent.companies lastObject];

    FHRemoteArray *facebookAccounts = _company.facebookAccounts;
    MTPocketRequest *request = [facebookAccounts next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    _facebookAccount = [facebookAccounts lastObject];
}

- (void)testOauthUrl
{
    NSURL *url = [FHFacebookAccount OAuthURLForCompany:_company withCallback:@"firehose://"];
    XCTAssertTrue([url absoluteString].length > 10);
}

- (void)testDestroyFacebookAccount
{
    MTPocketRequest *request = [_facebookAccount destroy];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}


@end
