//
//  FHInteractionTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/2/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "FHClient.h"
#import "macros.h"


@interface FHInteractionTests : XCTestCase
@property (nonatomic, strong) FHAgent       *agent;
@property (nonatomic, strong) FHCompany     *company;
@property (nonatomic, strong) FHCustomer    *customer;
@property (nonatomic, strong) FHInteraction *interaction;
@end


@implementation FHInteractionTests

+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent      = next_test_agent();
    _company    = [_agent.companies lastObject];

    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    _customer = [customers lastObject];

    FHRemoteArray *interactions = _customer.interactions;
    request = [interactions next];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    _interaction = [interactions lastObject];

    [interactions reset];
}



- (void)testIndex
{
    FHRemoteArray *interactions = _customer.interactions;
    MTPocketRequest *request = [interactions next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusSuccess);
    XCTAssertTrue(interactions.count > 0);
}

- (void)testReplyToTwitter
{
    MTPocketRequest *request = nil;

    _agent      = real_test_agent();
    _company    = [_agent.companies lastObject];

    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterTwitter
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    _customer = [customers lastObject];

    FHRemoteArray *interactions = _customer.interactions;
    request = [interactions next];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    FHInteraction *interaction = [interactions lastObject];
    interaction.responseDraft = [MTFaker lorem:10];
    request = [interaction reply];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
}

- (void)testReplyToFacebook
{
    MTPocketRequest *request = nil;

    _agent      = real_test_agent();
    _company    = [_agent.companies lastObject];

    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterFacebook
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    _customer = [customers lastObject];

    FHRemoteArray *interactions = _customer.interactions;
    request = [interactions next];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    FHInteraction *interaction = [interactions lastObject];
    interaction.responseDraft = [MTFaker lorem:10];
    request = [interaction reply];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
}

- (void)testReplyToEmail
{
    MTPocketRequest *request = nil;

    _agent      = real_test_agent();
    _company    = [_agent.companies lastObject];

    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterEmail
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    _customer = [customers lastObject];

    FHRemoteArray *interactions = _customer.interactions;
    request = [interactions next];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    FHInteraction *interaction = [interactions lastObject];
    interaction.responseDraft = [MTFaker lorem:10];
    request = [interaction reply];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
}

- (void)testAttachments
{
    MTPocketRequest *request = nil;

    _agent      = next_test_agent();
    _company    = [_agent.companies lastObject];

    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    for (FHCustomer *customer in customers) {
        FHRemoteArray *interactions = customer.interactions;
        request = [interactions next];

        resp  = nil;
        [request sendWithSuccess:^(MTPocketResponse *response) {
            resp = response;
        } failure:^(MTPocketResponse *response) {
            resp = response;
        }];

        STALL(resp == nil)
        XCTAssertTrue(resp.success);

        for (FHInteraction *interaction in interactions) {
            if ([interaction isKindOfClass:[FHEmailInteraction class]]) {
                FHEmailInteraction *emailInteraction = (FHEmailInteraction *)interaction;
                for (FHAttachment *attachment in emailInteraction.attachments) {
                    XCTAssertNotNil(attachment.identifier);
                    XCTAssertNotNil(attachment.filename);
                    XCTAssertNotNil(attachment.temporaryURL);
                    XCTAssertNotNil(attachment.createdAt);
                    return;
                }
            }
        }
    }
    XCTFail(@"Should not have reached here");
}

- (void)testUpdate
{
    _interaction.responseDraft  = [MTFaker lorem:10];
    _interaction.resolved       = !_interaction.resolved;

    MTPocketRequest *request = [_interaction save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testDestroy
{
    MTPocketRequest *request = [_interaction destroy];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testAddAndRemoveTag
{
    MTPocketRequest *request = nil;

    FHTag *tag = [FHTag tagForCompany:_company withLabel:[MTFaker lorem:1]];
    request = [tag save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    NSInteger tagCount = [_interaction.tags count];

    request = [_interaction addTag:tag];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
    XCTAssertTrue([_interaction.tags count] > tagCount);

    request = [_interaction removeTag:tag];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
    XCTAssertTrue([_interaction.tags count] == tagCount);
}

- (void)testFlagAndUnflagAgent
{
    MTPocketRequest *request = nil;

    // create a new agent to flag
    FHAgent *agent = [FHAgent agentWithEmail:[MTFaker email] password:[MTFaker email]];
    request = [agent signupWithFirstName:[MTFaker name] lastName:[MTFaker name]];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    // log the original agent back in
    request = [_agent login];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    NSInteger agentCount            = [_interaction.flaggedAgents count];
    NSInteger interactionAgentCount = [_interaction.customer.interactionFlaggedAgents count];

    // ad the agent to the company so we can flag them
    request = [_company addAgent:agent];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    // flag the agent
    request = [_interaction flagAgent:agent];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
    XCTAssertTrue([_interaction.flaggedAgents count] > agentCount);
    XCTAssertTrue([_interaction.customer.interactionFlaggedAgents count] > interactionAgentCount);

    // unflag the agent
    request = [_interaction unflagAgent:agent];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
    XCTAssertTrue([_interaction.flaggedAgents count] == agentCount);
    XCTAssertTrue([_interaction.customer.interactionFlaggedAgents count] == interactionAgentCount);
}


@end
