//
//  FHNoteTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/2/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "FHClient.h"
#import "macros.h"


@interface FHNoteTests : XCTestCase
@property (nonatomic, strong) FHAgent       *agent;
@property (nonatomic, strong) FHCompany     *company;
@property (nonatomic, strong) FHInteraction *interaction;
@property (nonatomic, strong) FHNote        *note;
@end


@implementation FHNoteTests

+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent = next_test_agent();
    _company = [_agent.companies lastObject];

    FHRemoteArray *customers = [_company customersMatchingResolvedFilter:FHCustomerResolvedFilterEverything
                                                           channelFilter:FHCustomerChannelFilterAll
                                                            searchString:nil
                                                                    sort:FHCustomerSortNewestFirst];
    MTPocketRequest *request = [customers next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    FHCustomer *customer = [customers lastObject];

    FHRemoteArray *interactions = customer.interactions;
    request = [interactions next];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    _interaction = [interactions lastObject];

    _note = [FHNote noteForInteraction:_interaction body:[MTFaker lorem:10]];
    request = [_note save];

    resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
}

- (void)testCreateNote
{
    NSInteger noteCount = [_interaction.notes count];

    FHNote *note = [FHNote noteForInteraction:_interaction body:[MTFaker lorem:10]];
    MTPocketRequest *request = [note save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
    XCTAssertTrue([_interaction.notes count] == noteCount + 1);
}

- (void)testUpdateNote
{
    _note.body = [MTFaker lorem:20];
    MTPocketRequest *request = [_note save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testDestroyNote
{
    NSInteger noteCount = [_interaction.notes count];

    MTPocketRequest *request = [_note destroy];
    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
    XCTAssertTrue([_interaction.notes count] == noteCount - 1);
}


@end
