//
//  FHTagTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/2/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "FHClient.h"
#import "macros.h"


@interface FHTagTests : XCTestCase
@property (nonatomic, strong) FHAgent   *agent;
@property (nonatomic, strong) FHCompany *company;
@property (nonatomic, strong) FHTag     *tag;
@end


@implementation FHTagTests

+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent      = next_test_agent();
    _company    = [_agent.companies lastObject];
    _tag        = [FHTag tagForCompany:_company withLabel:[MTFaker name]];
    MTPocketRequest *request = [_tag save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
}

- (void)testCreateTag
{
    FHTag *tag = [FHTag tagForCompany:_company withLabel:[MTFaker name]];
    MTPocketRequest *request = [tag save];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusCreated);
}

- (void)testUpdateTag
{
    MTPocketRequest *request = [_tag save];
    XCTAssertTrue(request.method == MTPocketMethodPUT);

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}

- (void)testDestroyTag
{
    MTPocketRequest *request = [_tag destroy];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}


@end



