//
//  FHTwitterAccountTests.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 2/2/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "FirehoseCocoa.h"
#import "MTFaker.h"
#import "FHClient.h"
#import "macros.h"


@interface FHTwitterAccountTests : XCTestCase
@property (nonatomic, strong) FHAgent          *agent;
@property (nonatomic, strong) FHCompany        *company;
@property (nonatomic, strong) FHTwitterAccount *twitterAccount;
@end


@implementation FHTwitterAccountTests

+ (void)setUp
{
    [[FHClient sharedClient] setEnvironment:ENV_ENUM];
}


- (void)setUp
{
    _agent      = next_test_agent();
    _company    = [_agent.companies lastObject];

    FHRemoteArray *twitterAccounts  = _company.twitterAccounts;
    MTPocketRequest *request        = [twitterAccounts next];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);

    _twitterAccount = [twitterAccounts lastObject];
}

- (void)testOauthUrl
{
    NSURL *url = [FHTwitterAccount OAuthURLForCompany:_company withCallback:@"firehose://"];
    XCTAssertTrue([url absoluteString].length > 10);
}

- (void)testDestroyTwitterAccount
{
    MTPocketRequest *request = [_twitterAccount destroy];

    __block MTPocketResponse *resp  = nil;
    [request sendWithSuccess:^(MTPocketResponse *response) {
        resp = response;
    } failure:^(MTPocketResponse *response) {
        resp = response;
    }];

    STALL(resp == nil)
    XCTAssertTrue(resp.success);
    XCTAssertTrue(resp.status == MTPocketStatusNoContent);
}


@end
