//
//  FHAgentInvite.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/26/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"


@interface FHAgentInvite ()
@property (nonatomic, assign, readwrite) FHCompany *company;
@property (nonatomic, copy,   readwrite) NSString  *email;
@end



@implementation FHAgentInvite


+ (FHAgentInvite *)agentInviteForCompany:(FHCompany *)company toEmail:(NSString *)email
{
    FHAgentInvite *agentInvite  = [FHAgentInvite agentInviteWithIdentifier:nil company:company];
    agentInvite.email           = email;
    return agentInvite;
}


- (MTPocketRequest *)save
{
    assert(self.identifier == nil);

    NSDictionary *body          = @{ @"agent_invite" : [self exportDictionary] };

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"companies/:company_id/agent_invites"
                                                            identifiers:@[self.company.identifier]
                                                                 method:MTPocketMethodPOST
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:body];
    [request addSuccess:^(MTPocketResponse *response) {
        [self populateWithDictionary:response.body];
        [(NSMutableOrderedSet *)_company.agentInvites addObject:self];
        self.isFetched = YES;
    }];

    return request;
}

- (MTPocketRequest *)resend
{
    assert(self.identifier != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"agent_invites/:id/resend"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodPUT
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    return request;
}

- (MTPocketRequest *)destroy
{
    assert(self.identifier != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"agent_invites/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_company.agentInvites removeObject:self];
    }];

    return request;
}




#pragma mark - Protected

+ (FHAgentInvite *)agentInviteWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company
{
    FHAgentInvite *agentInvite  = (FHAgentInvite *)[self objectWithIdentifier:identifier];
    agentInvite.company         = company;
    return agentInvite;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _email = [FHObject importString:dictionary[@"email"]];
    [super populateWithDictionary:dictionary];
}

- (NSDictionary *)exportDictionary
{
    [self exportString:_email as:@"email"];
    return [super exportDictionary];
}


@end
