//
//  FHAttachment.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/28/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"


@interface FHAttachment ()
@property (nonatomic, assign, readwrite) FHEmailInteraction *emailInteraction;
@end



@implementation FHAttachment


- (void)downloadWithProgress:(void (^)(float percent))progress
                     success:(void (^)(NSURL *localURL))success
                     failure:(void (^)(MTPocketResponse *response))failure
{
    NSString *filePath = [self resolvedLocalPath];

    // only download once
    if ([[NSFileManager defaultManager] fileExistsAtPath:filePath isDirectory:NULL]) {
        if (success) success([NSURL fileURLWithPath:filePath]);
        return;
    }

    NSURLRequest *request = [NSURLRequest requestWithURL:_temporaryURL];
    [NSURLConnection sendAsynchronousRequest:request
                                       queue:[NSOperationQueue mainQueue]
                           completionHandler:^(NSURLResponse *response, NSData *data, NSError *error)
    {
        if (!error) {
            [data writeToFile:filePath atomically:YES];
            if (success) success([NSURL fileURLWithPath:filePath]);
        }
    }];

//    MTPocketRequest *request = [MTPocketRequest requestWithURL:_temporaryURL
//                                                        method:MTPocketMethodGET
//                                                          body:nil];
//    [request sendWithSuccess:^(MTPocketResponse *response) {
//        [response.data writeToFile:filePath atomically:YES];
//        if (success) success([NSURL fileURLWithPath:filePath]);
//    } failure:^(MTPocketResponse *response) {
//        if (failure) failure(response);
//    } uploadProgress:nil downloadProgress:progress];
}

- (NSURL *)localURL
{
    return [NSURL fileURLWithPath:[self resolvedLocalPath]];
}




#pragma mark - Protected

+ (FHAttachment *)attachmentWithIdentifier:(NSNumber *)identifier emailInteraction:(FHEmailInteraction *)emailInteraction
{
    FHAttachment *attachment    = (FHAttachment *)[self objectWithIdentifier:identifier];
    attachment.emailInteraction = emailInteraction;
    return attachment;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _filename       = [FHObject importString:dictionary[@"filename"]];
    _temporaryURL   = [FHObject importURL:dictionary[@"temporary_url"]];
    [super populateWithDictionary:dictionary];
}




#pragma mark - Private

- (NSString *)resolvedLocalPath
{
    NSString *folderPath = [NSTemporaryDirectory() stringByAppendingString:[NSString stringWithFormat:@"%@", self.identifier]];
    if (![[NSFileManager defaultManager] fileExistsAtPath:folderPath isDirectory:NULL]) {
        [[NSFileManager defaultManager] createDirectoryAtPath:folderPath withIntermediateDirectories:YES attributes:nil error:nil];
    }
    return [folderPath stringByAppendingPathComponent:_filename];
}

@end


