//
//  FHCannedResponse.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/26/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"



@interface FHCannedResponse ()
@property (nonatomic, assign, readwrite) FHCompany *company;
@end



@implementation FHCannedResponse


+ (FHCannedResponse *)cannedResponseForCompany:(FHCompany *)company withName:(NSString *)name text:(NSString *)text
{
    FHCannedResponse *cannedResponse    = [FHCannedResponse cannedResponseWithIdentifier:nil company:company];
    cannedResponse.name                 = name;
    cannedResponse.text                 = text;
    return cannedResponse;
}

- (MTPocketRequest *)save
{
    MTPocketRequest *request    = nil;

    NSDictionary *body          = @{ @"canned_response" : [self exportDictionary] };

    // Create
    if (!self.identifier) {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"companies/:company_id/canned_responses"
                                               identifiers:@[self.company.identifier]
                                                    method:MTPocketMethodPOST
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
        [request addSuccess:^(MTPocketResponse *response) {
            [self populateWithDictionary:response.body];
            [(NSMutableOrderedSet *)_company.cannedResponses addObject:self];
            self.isFetched = YES;
        }];
    }

    // Update
    else {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"canned_responses/:id"
                                               identifiers:@[self.identifier]
                                                    method:MTPocketMethodPUT
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
    }

    return request;
}

- (MTPocketRequest *)destroy
{
    assert(self.identifier != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"canned_responses/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_company.cannedResponses removeObject:self];
    }];

    return request;
}




#pragma mark - Protected

+ (FHCannedResponse *)cannedResponseWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company
{
    FHCannedResponse *cannedResponse    = (FHCannedResponse *)[self objectWithIdentifier:identifier];
    cannedResponse.company              = company;
    return cannedResponse;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _name      = [FHObject importString:dictionary[@"name"]];
    _shortcut  = [FHObject importString:dictionary[@"shortcut"]];
    _text      = [FHObject importString:dictionary[@"text"]];
    [super populateWithDictionary:dictionary];
}

- (NSDictionary *)exportDictionary
{
    [self exportString:_name        as:@"name"];
    [self exportString:_shortcut    as:@"shortcut"];
    [self exportString:_text        as:@"text"];
    return [super exportDictionary];
}


@end
