//
//  FHClient.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/14/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//



#import "private.h"


@implementation FHClient

+ (FHClient *)sharedClient
{
    return (FHClient *)[FHClient sharedPocket];
}

- (void)setEnvironment:(FHEnvironment)environment
{
    _environment = environment;
    [FHClient sharedClient].defaultBaseURL = [self baseURLForServer:FHServerAPI];
    [self registerTemplates];
}

- (MTPocketRequest *)requestToServer:(FHServer)server
                               route:(NSString *)route
                         identifiers:(NSArray *)identifiers
                              method:(MTPocketMethod)method
                                page:(NSInteger)page
                             perPage:(NSInteger)perPage
                              params:(NSDictionary *)params
                                body:(id)body
{
    NSMutableDictionary *requestParams = [NSMutableDictionary new];
    if (page > 0 )      [requestParams addEntriesFromDictionary:@{ @"page"      : @(page)   }];
    if (perPage > 0)    [requestParams addEntriesFromDictionary:@{ @"per_page"  : @(perPage)}];
    [requestParams addEntriesFromDictionary:params];

    MTPocketRequest *request = nil;
    if (server == FHServerAPI) {
        request = [[FHClient sharedClient] requestWithTemplate:@"api"
                                                          path:route
                                                   identifiers:identifiers
                                                        method:method
                                                          body:body
                                                        params:requestParams];
        if (_token) [request addHeaders:[MTPocketRequest headerDictionaryForTokenAuthWithToken:_token]];
    }
    else if (server == FHServerBilling) {
        request = [[FHClient sharedClient] requestWithTemplate:@"billing"
                                                          path:route
                                                   identifiers:identifiers
                                                        method:method
                                                          body:body
                                                        params:requestParams];
        if (_billingToken) [request addHeaders:[MTPocketRequest headerDictionaryForTokenAuthWithToken:_billingToken]];
    }
    request.baseURL = [self baseURLForServer:server];
    return request;
}




#pragma mark - Private

- (void)registerTemplates
{
    MTPocketRequest *template   = [MTPocketRequest requestTemplate];
    template.format             = MTPocketFormatJSON;
    [[FHClient sharedClient] addTemplateWithRequest:template name:@"api"];

    template        = [MTPocketRequest requestTemplate];
    template.format = MTPocketFormatJSON;
    [[FHClient sharedClient] addTemplateWithRequest:template name:@"billing"];
}

- (NSURL *)baseURLForServer:(FHServer)server
{
    if (server == FHServerAPI) {
        switch (_environment) {
            case FHEnvironmentLocal:
                return [NSURL URLWithString:API_URL_LOCAL];
            case FHEnvironmentTunnel:
                return [NSURL URLWithString:API_URL_TUNNEL];
            case FHEnvironmentTesting:
                return [NSURL URLWithString:API_URL_TESTING];
            case FHEnvironmentProduction:
                return [NSURL URLWithString:API_URL_PRODUCTION];
            default:
                return [NSURL URLWithString:API_URL_LOCAL];
        }
    }
    else if (server == FHServerBilling) {
        switch (_environment) {
            case FHEnvironmentLocal:
                return [NSURL URLWithString:BILLING_URL_PRODUCTION];
            case FHEnvironmentTunnel:
                return [NSURL URLWithString:BILLING_URL_PRODUCTION];
            case FHEnvironmentTesting:
                return [NSURL URLWithString:BILLING_URL_PRODUCTION];
            case FHEnvironmentProduction:
                return [NSURL URLWithString:BILLING_URL_PRODUCTION];
            default:
                return [NSURL URLWithString:BILLING_URL_PRODUCTION];
        }
    }
    return [NSURL URLWithString:API_URL_LOCAL];
}




@end
