//
//  FHCompany.h
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/14/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "FHObject.h"

@class FHCreditCard;


typedef NS_ENUM(NSInteger, FHCustomerResolvedFilter) {
    FHCustomerResolvedFilterEverything,
    FHCustomerResolvedFilterUnresolved
};

typedef NS_OPTIONS(NSUInteger, FHCustomerChannelFilter) {
    FHCustomerChannelFilterTwitter  = 1UL << 1,
    FHCustomerChannelFilterFacebook = 1UL << 2,
    FHCustomerChannelFilterEmail    = 1UL << 3,
    FHCustomerChannelFilterAll      = FHCustomerChannelFilterTwitter | FHCustomerChannelFilterFacebook | FHCustomerChannelFilterEmail
};

typedef NS_ENUM(NSInteger, FHCustomerSort) {
    FHCustomerSortNewestFirst,
    FHCustomerSortOldestFirst
};




@interface FHCompany : FHObject

@property (nonatomic, copy          ) NSString     *title;
@property (nonatomic, copy          ) NSString     *token;
@property (nonatomic, copy, readonly) NSDate       *lastFetchAt;
@property (nonatomic                ) BOOL         fetchAutomatically;
@property (nonatomic,       readonly) NSInteger    unresolvedCount;
@property (nonatomic,       readonly) NSInteger    numberOfAccounts;

@property (nonatomic, copy, readonly) NSOrderedSet *agents;
@property (nonatomic, copy, readonly) NSOrderedSet *agentInvites;
@property (nonatomic, copy, readonly) NSOrderedSet *tags;
@property (nonatomic, copy, readonly) NSOrderedSet *cannedResponses;


+ (FHCompany *)companyWithAgent:(FHAgent *)agent title:(NSString *)title;
- (MTPocketRequest *)fetch;                                                    // Will fetch the company and all associations
- (MTPocketRequest *)save;                                                     // Will create or update depending on if it already exists.
- (MTPocketRequest *)forceChannelsFetch;
- (MTPocketRequest *)destroy;                                                  // Will render all dependent objects invalid.


@property (nonatomic, strong, readonly) FHRemoteArray *notifications;
@property (nonatomic, strong, readonly) FHRemoteArray *twitterAccounts;
@property (nonatomic, strong, readonly) FHRemoteArray *facebookAccounts;
@property (nonatomic, strong, readonly) FHRemoteArray *emailAccounts;

/**
 The `searchString` can contain any combination of space deliminated tokens representing:
    1. Plain text to search for within customer interactions.
    2. Tags, prefixed with a '#', to filter customers with interactions with that tag.
    3. Agent names, prefixed with '@', capitalized, with first and last name seperated by an underscore, to
        filter on customers that have interactions flagged for that agent.
 */
- (FHRemoteArray *)customersMatchingResolvedFilter:(FHCustomerResolvedFilter)resolvedFilter
                                     channelFilter:(FHCustomerChannelFilter)channelFilter
                                      searchString:(NSString *)searchString
                                              sort:(FHCustomerSort)sort;

- (MTPocketRequest *)addAgent:(FHAgent *)agent;
- (MTPocketRequest *)removeAgent:(FHAgent *)agent;


// TODO: write tests for this
/**
 You must call `checkIfCreditCardIsAdded` before this property is available. Until it is called
 the value of this property is nil. Once it is called, you can use it as [... boolValue].
 */
@property (nonatomic, strong) FHCreditCard *creditCard;
@property (nonatomic, strong) NSString     *billingEmail;
@property (nonatomic, assign) CGFloat      billingRate;
@property (nonatomic, strong) NSDate       *trialExpirationDate;
@property (nonatomic, strong) NSDate       *nextBillingDate;
@property (nonatomic, assign) BOOL         isGracePeriodOver;
@property (nonatomic, assign) NSInteger    daysLeftInGracePeriod;
@property (nonatomic, assign) BOOL         isCurrent;
@property (nonatomic, assign) BOOL         hasSuccessfulBilling;
- (MTPocketRequest *)fetchBillingInfo;

@end

