//
//  FHCustomer.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/15/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"


@interface FHCustomer ()
@property (nonatomic, assign, readwrite) FHCompany     *company;
@property (nonatomic, strong, readwrite) FHRemoteArray *interactions;
@end


@implementation FHCustomer


- (id)init
{
    self = [super init];
    if (self) {
        _customerAccounts           = [NSMutableOrderedSet orderedSet];
        _interactionFlaggedAgents   = [NSMutableOrderedSet orderedSet];
    }
    return self;
}


+ (FHCustomer *)customerWithIdentifier:(NSNumber *)identifier
{
    return (FHCustomer *)[self objectWithIdentifier:identifier];
}

- (MTPocketRequest *)fetch
{
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"customers/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodGET
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [self populateWithDictionary:response.body];
        self.isFetched = YES;
    }];

    return request;
}

- (MTPocketRequest *)destroy
{
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"customers/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    return request;
}

- (MTPocketRequest *)resolveAllInteractions
{
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"customers/:id/resolve_all_interactions"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodPUT
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    return request;
}





#pragma mark - Associations

- (FHRemoteArray *)interactions
{
    assert(self.identifier);
    if (!_interactions) {
        _interactions = [FHRemoteArray remoteArrayWithPath:@"customers/:id/interactions" identifiers:@[self.identifier] params:nil createObjectBlock:^id(NSDictionary *dict) {
            return [FHInteraction interactionWithDictionary:dict customer:self];
        }];
    }
    return _interactions;
}




#pragma mark - Protected

+ (FHCustomer *)customerWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company
{
    FHCustomer *customer = [FHCustomer customerWithIdentifier:identifier];
    customer.company = company;
    return customer;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _name                           = [FHObject importString:dictionary[@"name"]];
    _location                       = [FHObject importString:dictionary[@"location"]];
    _timeZone                       = [FHObject importString:dictionary[@"time_zone"]];
    _newestInteractionIdentifier    = [FHObject importNumber:dictionary[@"newest_interaction_id"]];
    _newestInteractionExcerpt       = [FHObject importString:dictionary[@"newest_interaction_excerpt"]];
    _newestInteractionReceivedAt    = [FHObject importDate:dictionary[@"newest_interaction_received_at"]];

    [self populateAssociatedObjects:&_customerAccounts withDictionaries:dictionary[@"customer_accounts"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHCustomerAccount customerAccountWithIdentifier:dict[@"id"] customer:self];
    }];

    [self populateAssociatedObjects:&_interactionFlaggedAgents withDictionaries:dictionary[@"interaction_flagged_agents"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHAgent agentWithIdentifier:dict[@"id"]];
    }];

    [self populateAssociatedObject:&_agentWithDibs withDictionary:dictionary[@"agent_with_dibs"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHAgent agentWithIdentifier:dict[@"id"]];
    }];
    
    
    [super populateWithDictionary:dictionary];
}


@end
