//
//  FHEmailAccount.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/26/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"



@interface FHEmailAccount ()
@property (nonatomic, assign, readwrite) FHCompany *company;
@property (nonatomic, copy             ) NSArray   *popularServices;
@end



@implementation FHEmailAccount {
    NSArray *_popularServices;
}


+ (FHEmailAccount *)emailAccountForCompany:(FHCompany *)company
                                      kind:(FHEmailAccountKind)kind
                                    server:(NSString *)server
                                      port:(NSInteger)port
                                  username:(NSString *)username
                                  password:(NSString *)password
                                  usingSSL:(BOOL)SSL
{
    FHEmailAccount *emailAccount    = [FHEmailAccount emailAccountWithIdentifier:nil company:company];
    emailAccount.kind               = kind;
    emailAccount.server             = server;
    emailAccount.port               = port;
    emailAccount.username           = username;
    emailAccount.password           = password;
    emailAccount.SSL                = SSL;
    return emailAccount;
}

- (MTPocketRequest *)save
{
    MTPocketRequest *request = nil;

    NSDictionary *body = @{ @"email_account" : [self exportDictionary] };

    // Create
    if (!self.identifier) {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"companies/:company_id/email_accounts"
                                               identifiers:@[self.company.identifier]
                                                    method:MTPocketMethodPOST
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
        [request addSuccess:^(MTPocketResponse *response) {
            [self populateWithDictionary:response.body];
            [(NSMutableOrderedSet *)_company.emailAccounts addObject:self];
            self.isFetched = YES;
        }];
    }

    // Update
    else {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"email_accounts/:id"
                                               identifiers:@[self.identifier]
                                                    method:MTPocketMethodPUT
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
    }

    return request;
}

- (MTPocketRequest *)destroy
{
    assert(self.identifier != nil);

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"email_accounts/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_company.emailAccounts removeObject:self];
    }];

    return request;
}

- (BOOL)guessFieldsFromEmail
{
    for (NSDictionary *service in [self popularServices]) {
        NSString *domain = [NSString stringWithFormat:@"@%@", service[@"Domain"]];
        if ([_username rangeOfString:domain].location != NSNotFound) {
            _kind   = [service[@"Kind"] integerValue];
            _SSL    = [service[@"SSL"] boolValue];
            _port   = [service[@"Port"] integerValue];
            _server = service[@"Server"];
            return YES;
        }
    }
    return NO;
}





#pragma mark - Protected

+ (FHEmailAccount *)emailAccountWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company
{
    FHEmailAccount *emailAccount    = (FHEmailAccount *)[self objectWithIdentifier:identifier];
    emailAccount.company            = company;
    return emailAccount;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _server             = [FHObject importString:dictionary[@"incoming_server"]];
    _SSL                = [FHObject importBool:dictionary[@"incoming_ssl"]];
    _port               = [FHObject importInteger:dictionary[@"incoming_port"]];
    _username           = [FHObject importString:dictionary[@"incoming_username"]];
    _kind               = [FHObject importEnum:dictionary[@"kind"] usingMap:[FHEmailAccount emailAccountEnumMap]];
    _deleteFromServer   = [FHObject importBool:dictionary[@"delete_from_server"]];
    [super populateWithDictionary:dictionary];
}

+ (NSDictionary *)emailAccountEnumMap
{
    return @{
             @"IMAP" : @(FHEmailAccountKindIMAP),
             @"POP"  : @(FHEmailAccountKindPOP)
             };
}

- (NSDictionary *)exportDictionary
{
    [self exportString:_server          as:@"incoming_server"];
    [self exportBool:_SSL               as:@"incoming_ssl"];
    [self exportInteger:_port           as:@"incoming_port"];
    [self exportString:_username        as:@"incoming_username"];
    [self exportString:_password        as:@"incoming_password"];
    [self exportEnum:_kind              as:@"kind" usingMap:[FHEmailAccount emailAccountEnumMap]];
    [self exportBool:_deleteFromServer  as:@"delete_from_server"];
    return [super exportDictionary];
}




#pragma mark - Private

- (NSArray *)popularServices
{
    if (!_popularServices) {
        _popularServices =  @[
                              @{
                                  @"Domain"  : @"gmail.com",
                                  @"Kind"    : @(FHEmailAccountKindIMAP),
                                  @"SSL"     : @(YES),
                                  @"Port"    : @(993),
                                  @"Server"  : @"imap.googlemail.com"
                                  },
                              @{
                                  @"Domain"  : @"hotmail.com",
                                  @"Kind"    : @(FHEmailAccountKindPOP),
                                  @"SSL"     : @(YES),
                                  @"Port"    : @(995),
                                  @"Server"  : @"pop3.live.com"
                                  },
                              @{
                                  @"Domain"  : @"aol.com",
                                  @"Kind"    : @(FHEmailAccountKindIMAP),
                                  @"SSL"     : @(YES),
                                  @"Port"    : @(993),
                                  @"Server"  : @"imap.aol.com"
                                  },
                              @{
                                  @"Domain"  : @"aim.com",
                                  @"Kind"    : @(FHEmailAccountKindIMAP),
                                  @"SSL"     : @(YES),
                                  @"Port"    : @(993),
                                  @"Server"  : @"imap.aim.com"
                                  },
                              @{
                                  @"Domain"  : @"me.com",
                                  @"Kind"    : @(FHEmailAccountKindIMAP),
                                  @"SSL"     : @(YES),
                                  @"Port"    : @(993),
                                  @"Server"  : @"imap.mail.me.com"
                                  },
                              @{
                                  @"Domain"  : @"mac.com",
                                  @"Kind"    : @(FHEmailAccountKindIMAP),
                                  @"SSL"     : @(YES),
                                  @"Port"    : @(993),
                                  @"Server"  : @"imap.mail.me.com"
                                  },
                              @{
                                  @"Domain"  : @"icloud.com",
                                  @"Kind"    : @(FHEmailAccountKindIMAP),
                                  @"SSL"     : @(YES),
                                  @"Port"    : @(993),
                                  @"Server"  : @"imap.mail.me.com"
                                  },
                              @{
                                  @"Domain"  : @"yahoo.com",
                                  @"Kind"    : @(FHEmailAccountKindIMAP),
                                  @"SSL"     : @(YES),
                                  @"Port"    : @(993),
                                  @"Server"  : @"imap.mail.yahoo.com"
                                  }
                              ];
    }
    return _popularServices;
}


@end
