//
//  FHFacebookAccount.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/26/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"



@interface FHFacebookAccount ()
@property (nonatomic, assign, readwrite) FHCompany *company;
@end



@implementation FHFacebookAccount


- (id)init
{
    self = [super init];
    if (self) {
        _facebookPages = [NSMutableOrderedSet orderedSet];
    }
    return self;
}


+ (NSURL *)OAuthURLForCompany:(FHCompany *)company withCallback:(NSString *)callback
{
    assert(company != nil);
    assert(callback != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestWithTemplate:@"api" path:@"companies/:company_id/oauth_facebook" identifiers:@[company.identifier] method:MTPocketMethodGET body:nil params:@{ @"url_token" : [FHClient sharedClient].URLToken, @"callback_url" : callback }];
    return [request resolvedURL];
}

- (MTPocketRequest *)destroy
{
    assert(self.identifier != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"facebook_accounts/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_company.facebookAccounts removeObject:self];
        self.isFetched = YES;
    }];

    return request;
}



#pragma mark - Protected

+ (FHFacebookAccount *)facebookAccountWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company
{
    FHFacebookAccount *facebookAccount = (FHFacebookAccount *)[self objectWithIdentifier:identifier];
    facebookAccount.company = company;
    return facebookAccount;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _username       = [FHObject importString:dictionary[@"username"]];
    _facebookUserId = [FHObject importNumber:dictionary[@"facebook_user_id"]];
    _imageURL       = [FHObject importURL:dictionary[@"image_url"]];
    _name           = [FHObject importString:dictionary[@"name"]];

    [self populateAssociatedObjects:&_facebookPages withDictionaries:dictionary[@"facebook_pages"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHFacebookPage facebookPageWithIdentifier:dict[@"id"] facebookAccount:self];
    }];
    
    [super populateWithDictionary:dictionary];
}


@end
