//
//  FHFacebookInteraction.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/28/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"

@implementation FHFacebookInteraction


- (NSString *)typeString
{
    NSDictionary *map = [FHFacebookInteraction postTypeEnumMap];
    for (NSString *key in [map allKeys]) {
        NSNumber *value = map[key];
        if ([value integerValue] == _postType) {
            return key;
        }
    }
    return @"status";
}



#pragma mark - Protected

+ (FHFacebookInteraction *)facebookInteractionWithIdentifier:(NSNumber *)identifier
{
    return (FHFacebookInteraction *)[self objectWithIdentifier:identifier];
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    NSDictionary *facebookInteractionDict   = dictionary[@"facebook_interaction"];
    _fromUserId     = [FHObject importNumber:facebookInteractionDict[@"from_user_id"]];
    _fromName       = [FHObject importString:facebookInteractionDict[@"from_name"]];
    _toUserId       = [FHObject importNumber:facebookInteractionDict[@"to_user_id"]];
    _toName         = [FHObject importString:facebookInteractionDict[@"to_name"]];
    _postId         = [FHObject importString:facebookInteractionDict[@"post_id"]];
    _commentId      = [FHObject importString:facebookInteractionDict[@"comment_id"]];
    _postType       = [FHObject importEnum:facebookInteractionDict[@"post_type"] usingMap:[FHFacebookInteraction postTypeEnumMap]];
    _postExcerpt    = [FHObject importString:facebookInteractionDict[@"post_excerpt"]];
    _likeCount      = [facebookInteractionDict[@"like_count"] integerValue];
    [super populateWithDictionary:dictionary];
}

+ (NSDictionary *)postTypeEnumMap
{
    return @{
             @"status"   : @(FHFacebookPostTypeStatus),
             @"photo"    : @(FHFacebookPostTypePhoto),
             @"comment"  : @(FHFacebookPostTypeComment),
             @"video"    : @(FHFacebookPostTypeVideo),
             @"link"     : @(FHFacebookPostTypeLink)
             };
}


@end
