//
//  FHInteraction.h
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/21/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//
@class FHCustomerAccount, FHAgent, FHTag;

#import "FHObject.h"


typedef enum {
    FHInteractionChannelEmail,
    FHInteractionChannelTwitter,
    FHInteractionChannelFacebook
} FHInteractionChannel;

typedef enum {
    FHInteractionHappinessUnknown       = -1,
    FHInteractionHappinessUpset         = 0,
    FHInteractionHappinessSatisfied     = 1,
    FHInteractionHappinessHappy         = 2
} FHInteractionHappiness;




@interface FHInteraction : FHObject

@property (nonatomic, assign, readonly) FHCustomer             *customer;

@property (nonatomic, copy,   readonly) NSString               *token;
@property (nonatomic, copy            ) NSString               *responseDraft;
@property (nonatomic,         readonly) FHInteractionHappiness happiness;
@property (nonatomic                  ) BOOL                   resolved;

@property (nonatomic, copy,   readonly) NSString               *subject;
@property (nonatomic, copy,   readonly) NSString               *body;
@property (nonatomic, copy,   readonly) NSURL                  *privateURL;
@property (nonatomic,         readonly) FHInteractionChannel   channel;
@property (nonatomic, copy,   readonly) NSDate                 *receivedAt;
@property (nonatomic, copy,   readonly) NSDate                 *createdAt;

@property (nonatomic, strong, readonly) FHCustomerAccount      *customerAccount;
@property (nonatomic, strong, readonly) FHAgent                *agent;

@property (nonatomic, copy,   readonly) NSOrderedSet           *responseInteractions;
@property (nonatomic, copy,   readonly) NSOrderedSet           *notes;
@property (nonatomic, copy,   readonly) NSOrderedSet           *tags;
@property (nonatomic, copy,   readonly) NSOrderedSet           *flaggedAgents;

// set the responseDraft property before calling this. This will reply with its contents.
- (MTPocketRequest *)reply;
- (MTPocketRequest *)save;
- (MTPocketRequest *)destroy;

- (MTPocketRequest *)addTag:(FHTag *)tag;
- (MTPocketRequest *)removeTag:(FHTag *)tag;

- (MTPocketRequest *)flagAgent:(FHAgent *)agent;
- (MTPocketRequest *)unflagAgent:(FHAgent *)agent;


- (NSString *)happinessString;

@end
