//
//  FHInteraction.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/21/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"


@implementation FHInteraction


#pragma mark - Protected

- (id)init
{
    self = [super init];
    if (self) {
        _happiness              = -1;
        _resolved               = NO;
        _responseInteractions   = [NSMutableOrderedSet orderedSet];
        _notes                  = [NSMutableOrderedSet orderedSet];
        _tags                   = [NSMutableOrderedSet orderedSet];
        _flaggedAgents          = [NSMutableOrderedSet orderedSet];
    }
    return self;
}




#pragma mark - Public

- (NSString *)subject
{
    if ([self isKindOfClass:[FHEmailInteraction class]]) {
        FHEmailInteraction *emailInteraction = (FHEmailInteraction *)self;
        return emailInteraction.emailSubject;
    }
    else if ([self isKindOfClass:[FHTwitterInteraction class]]) {
        FHTwitterInteraction *twitterInteraction = (FHTwitterInteraction *)self;
        return twitterInteraction.inReplyToScreenName ?
        [NSString stringWithFormat:@"Reply to @%@", twitterInteraction.inReplyToScreenName] :
        [NSString stringWithFormat:@"Mention of @%@", twitterInteraction.toScreenName];
    }
    else if ([self isKindOfClass:[FHFacebookInteraction class]]) {
        FHFacebookInteraction *facebookInteraction = (FHFacebookInteraction *)self;
        return [[facebookInteraction typeString] capitalizedString];
    }
    return @"";
}

- (MTPocketRequest *)reply
{
    assert(self.identifier != nil);
    assert(_responseDraft != nil);

    MTPocketRequest *request = nil;

    NSDictionary *body = @{ @"interaction" : @{ @"body" : _responseDraft } };
    request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                 route:@"interactions/:id/reply"
                                           identifiers:@[self.identifier]
                                                method:MTPocketMethodPOST
                                                  page:-1
                                               perPage:-1
                                                params:nil
                                                  body:body];
    [request addSuccess:^(MTPocketResponse *response) {
        self.responseDraft                      = nil;
        FHCustomer *customer                    = self.customer;

        // update with latest info from server and make sure sending agent gets set
        __block FHInteraction *newInteraction   = nil;
        [self addToAssociatedObjects:_responseInteractions withDictionary:response.body createObjectBlock:^id(NSDictionary *dict) {
            FHInteraction *interaction = [FHInteraction interactionWithDictionary:dict customer:customer];
            newInteraction = interaction;
            return interaction;
        }];
        newInteraction.agent = [FHAgent loggedInAgent];

        // sort responses
        [(NSMutableOrderedSet *)_responseInteractions sortUsingComparator:^NSComparisonResult(FHInteraction *interaction1, FHInteraction *interaction2) {
            return [interaction1.createdAt mt_isAfter:interaction2.createdAt] ? NSOrderedAscending : NSOrderedDescending;
        }];
    }];

    return request;
}

- (MTPocketRequest *)save
{
    assert(self.identifier);
    MTPocketRequest *request = nil;

    NSDictionary *body = @{ @"interaction" : [self exportDictionary] };
    request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                 route:@"interactions/:id"
                                           identifiers:@[self.identifier]
                                                method:MTPocketMethodPUT
                                                  page:-1
                                               perPage:-1
                                                params:nil
                                                  body:body];
    return request;
}

- (MTPocketRequest *)destroy
{
    assert(self.identifier);
    MTPocketRequest *request = nil;

    request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                 route:@"interactions/:id"
                                           identifiers:@[self.identifier]
                                                method:MTPocketMethodDELETE
                                                  page:-1
                                               perPage:-1
                                                params:nil
                                                  body:nil];

    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_customer.interactions removeObject:self];
    }];

    return request;
}

- (MTPocketRequest *)addTag:(FHTag *)tag
{
    assert(self.identifier);
    assert(tag.identifier);

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"interactions/:interaction_id/tags/:id"
                                                            identifiers:@[self.identifier, tag.identifier]
                                                                 method:MTPocketMethodPUT
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_tags addObject:tag];
    }];
    return request;
}

- (MTPocketRequest *)removeTag:(FHTag *)tag
{
    assert(self.identifier);
    assert(tag.identifier);

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"interactions/:interaction_id/tags/:id"
                                                            identifiers:@[self.identifier, tag.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_tags removeObject:tag];
    }];
    return request;
}


- (MTPocketRequest *)flagAgent:(FHAgent *)agent
{
    assert(self.identifier);
    assert(agent.identifier);

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"interactions/:interaction_id/agents/:id"
                                                            identifiers:@[self.identifier, agent.identifier]
                                                                 method:MTPocketMethodPUT
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_flaggedAgents addObject:agent];
        [(NSMutableOrderedSet *)_customer.interactionFlaggedAgents addObject:agent];
    }];
    return request;
}

- (MTPocketRequest *)unflagAgent:(FHAgent *)agent
{
    assert(self.identifier);
    assert(agent.identifier);

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"interactions/:interaction_id/agents/:id"
                                                            identifiers:@[self.identifier, agent.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_flaggedAgents removeObject:agent];
        [(NSMutableOrderedSet *)_customer.interactionFlaggedAgents removeObject:agent];
    }];
    return request;
}

- (NSString *)happinessString
{
    if (_happiness == FHInteractionHappinessHappy)
        return @"Happy";
    else if (_happiness == FHInteractionHappinessUpset)
        return @"Upset";
    else if (_happiness == FHInteractionHappinessSatisfied)
        return @"Satisfied";

    return @"Unknown";
}




#pragma mark - Protected

+ (FHInteraction *)interactionWithDictionary:(NSDictionary *)dictionary customer:(FHCustomer *)customer
{
    FHInteraction *interaction = nil;

    FHInteractionChannel channel = [FHObject importEnum:dictionary[@"channel"] usingMap:[FHInteraction channelEnumMap]];
    if (channel == FHInteractionChannelTwitter) {
        interaction = [FHTwitterInteraction twitterInteractionWithIdentifier:dictionary[@"id"]];
        interaction.customer = customer;
    }
    else if (channel == FHInteractionChannelFacebook) {
        interaction = [FHFacebookInteraction facebookInteractionWithIdentifier:dictionary[@"id"]];
        interaction.customer = customer;
    }
    else if (channel == FHInteractionChannelEmail) {
        interaction = [FHEmailInteraction emailInteractionWithIdentifier:dictionary[@"id"]];
        interaction.customer = customer;
    }

    [interaction populateWithDictionary:dictionary];

    return interaction;
}


- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _token              = [FHObject importString:dictionary[@"token"]];
    _body               = [FHObject importString:dictionary[@"body"]];
    _responseDraft      = [FHObject importString:dictionary[@"response_draft"]];
    _channel            = [FHObject importEnum:dictionary[@"channel"] usingMap:[FHInteraction channelEnumMap]];
    _receivedAt         = [FHObject importDate:dictionary[@"received_at"]];
    _privateURL         = [FHObject importURL:dictionary[@"private_url"]];
    _happiness          = (int)[FHObject importInteger:dictionary[@"happiness"]];
    _resolved           = [FHObject importBool:dictionary[@"resolved"]];

    [self populateAssociatedObject:&_agent withDictionary:dictionary[@"agent"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHAgent agentWithIdentifier:dict[@"id"]];
    }];

    [self populateAssociatedObject:&_customerAccount withDictionary:dictionary[@"customer_account"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHCustomerAccount customerAccountWithIdentifier:dict[@"id"] customer:self.customer];
    }];

    [self populateAssociatedObjects:&_responseInteractions withDictionaries:dictionary[@"response_interactions"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHInteraction interactionWithDictionary:dict customer:self.customer];
    }];

    [self populateAssociatedObjects:&_notes withDictionaries:dictionary[@"notes"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHNote noteWithIdentifier:dict[@"id"] interaction:self];
    }];

    [self populateAssociatedObjects:&_tags withDictionaries:dictionary[@"tags"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHTag tagWithIdentifier:dict[@"id"] company:self.customer.company];
    }];

    [self populateAssociatedObjects:&_flaggedAgents withDictionaries:dictionary[@"flagged_agents"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHAgent agentWithIdentifier:dict[@"id"]];
    }];

    [super populateWithDictionary:dictionary];
}


- (NSDictionary *)exportDictionary
{
    [self exportBool:_resolved          as:@"resolved"];
    [self exportString:_responseDraft   as:@"response_draft"];
    return [super exportDictionary];
}




#pragma mark - Private

+ (NSDictionary *)channelEnumMap
{
    return @{
             @"twitter"   : @(FHInteractionChannelTwitter),
             @"facebook"  : @(FHInteractionChannelFacebook),
             @"email"     : @(FHInteractionChannelEmail)
             };
}

@end
