//
//  FHNote.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/26/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"


@interface FHNote ()
@property (nonatomic, assign, readwrite) FHInteraction *interaction;
@end



@implementation FHNote


+ (FHNote *)noteForInteraction:(FHInteraction *)interaction body:(NSString *)body
{
    FHNote *note    = [FHNote noteWithIdentifier:nil interaction:interaction];
    note.body       = body;
    return note;
}

- (MTPocketRequest *)save
{
    MTPocketRequest *request    = nil;
    NSDictionary *body          = @{ @"note" : [self exportDictionary] };

    if (!self.identifier) {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"interactions/:interaction_id/notes"
                                               identifiers:@[self.interaction.identifier]
                                                    method:MTPocketMethodPOST
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
        [request addSuccess:^(MTPocketResponse *response) {
            [self populateWithDictionary:response.body];
            [(NSMutableOrderedSet *)_interaction.notes addObject:self];
            [(NSMutableOrderedSet *)_interaction.notes sortUsingComparator:^NSComparisonResult(FHNote *note1, FHNote *note2) {
                return [note1.createdAt mt_isAfter:note2.createdAt] ? NSOrderedAscending : NSOrderedDescending;
            }];
            self.isFetched = YES;
        }];
    }
    else {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"notes/:id"
                                               identifiers:@[self.identifier]
                                                    method:MTPocketMethodPUT
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
    }
    return request;
}

- (MTPocketRequest *)destroy
{
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"notes/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_interaction.notes removeObject:self];
    }];
    return request;
}




#pragma mark - Protected

+ (FHNote *)noteWithIdentifier:(NSNumber *)identifier interaction:(FHInteraction *)interaction
{
    FHNote *note        = (FHNote *)[self objectWithIdentifier:identifier];
    note.interaction    = interaction;
    return note;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _body   = [FHObject importString:dictionary[@"body"]];

    [self populateAssociatedObject:&_agent withDictionary:dictionary[@"agent"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHAgent agentWithIdentifier:dict[@"id"]];
    }];

    [super populateWithDictionary:dictionary];
}

- (NSDictionary *)exportDictionary
{
    [self exportString:_body as:@"body"];
    return [super exportDictionary];
}


@end
