//
//  FHOutgoingAttachment.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 8/22/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"

#if TARGET_OS_IPHONE
#import <MobileCoreServices/UTType.h>
#import <UIKit/UIDevice.h>
#else
#import <CoreServices/CoreServices.h>
#endif




@interface FHOutgoingAttachment ()
@property (nonatomic, strong, readwrite) NSString *filename;
@property (nonatomic, strong, readwrite) NSString *MIMEType;
@property (nonatomic, strong, readwrite) NSString *token;
@property (nonatomic, strong, readwrite) NSURL    *downloadURL;
@property (nonatomic, strong, readwrite) NSURL    *uploadURL;
@property (nonatomic, assign, readwrite) BOOL     uploaded;
@property (nonatomic, strong, readwrite) NSURL    *localURL;
@end




@implementation FHOutgoingAttachment

+ (FHOutgoingAttachment *)outgoingAttachmentWithLocalURL:(NSURL *)localURL
{
    NSString *localPath                         = [localURL absoluteString];
    FHOutgoingAttachment *outgoingAttachment    = [FHOutgoingAttachment new];
    outgoingAttachment.filename                 = [localPath lastPathComponent];
    outgoingAttachment.MIMEType                 = [self MIMETypeFromPathExtension:[localPath pathExtension]];
    outgoingAttachment.uploaded                 = NO;
    outgoingAttachment.localURL                 = localURL;
    return outgoingAttachment;
}

- (void)uploadWithProgress:(void (^)(float percent))progress
                   success:(void (^)(NSURL *downloadURL))success
                   failure:(void (^)(MTPocketResponse *response))failure
{
    NSDictionary *body = @{ @"outgoing_attachment" : [self exportDictionary] };

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"companies/:company_id/outgoing_attachments"
                                                            identifiers:@[[FHAgent loggedInAgent].currentCompany.identifier]
                                                                 method:MTPocketMethodPOST
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:body];
    [request addSuccess:^(MTPocketResponse *response) {
        [self populateWithDictionary:response.body];
    }];

    [request sendWithSuccess:^(MTPocketResponse *response) {
        NSData *data = [NSData dataWithContentsOfURL:_localURL];
        MTPocketRequest *request = [MTPocketRequest requestWithURL:_uploadURL
                                                            method:MTPocketMethodPUT
                                                              body:data];
        request.headers[@"Accept"]          = @"*/*";
        request.headers[@"Content-Type"]    = _MIMEType;
        request.headers[@"x-amz-acl"]       = @"authenticated-read";
        request.headers[@"Connection"]       = @"keep-alive";
        request.headers[@"Content-Length"]       = [NSString stringWithFormat:@"%lu", (unsigned long)[data length]];
        //        request.headers[@"Access-Control-Request-Headers"]       = @"x-amz-acl, origin, content-type";
//        request.headers[@"Access-Control-Request-Method"] = @"PUT";
//        request.headers[@"Origin"] = @"https://firehoseapp.com";


        [request sendWithSuccess:^(MTPocketResponse *response) {
            if (success) success(_downloadURL);
            NSDictionary *body = @{ @"outgoing_attachment" : @{ @"uploaded" : @YES } };
            MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                          route:@"outgoing_attachments/:outgoing_attachment_id"
                                                                    identifiers:@[self.identifier]
                                                                         method:MTPocketMethodPUT
                                                                           page:-1
                                                                        perPage:-1
                                                                         params:nil
                                                                           body:body];
            [request sendWithSuccess:nil failure:^(MTPocketResponse *response) {
                if (failure) failure(response);
            }];
        } failure:^(MTPocketResponse *response) {
            if (failure) failure(response);
        } uploadProgress:^(float percent) {
            if (progress) progress(percent);
        } downloadProgress:^(float percent) {

        }];

    } failure:^(MTPocketResponse *response) {
        if (failure) failure(response);
    }];

}



#pragma mark - Protected

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _token          = [FHObject importString:dictionary[@"token"]];
    _downloadURL    = [FHObject importURL:dictionary[@"download_url"]];
    _uploadURL      = [FHObject importURL:dictionary[@"upload_url"]];
    _uploaded       = [FHObject importBool:dictionary[@"uploaded"]];
    _filename       = [FHObject importString:dictionary[@"filename"]];
    _MIMEType       = [FHObject importString:dictionary[@"mimetype"]];

    [super populateWithDictionary:dictionary];
}

- (NSDictionary *)exportDictionary
{
    [self exportString:_filename as:@"filename"];
    [self exportString:_MIMEType as:@"mimetype"];
    return [super exportDictionary];
}





#pragma mark - Private

+ (NSString *)MIMETypeFromPathExtension:(NSString *)pathExtension
{
    CFStringRef uti = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, (__bridge CFStringRef)pathExtension, NULL);
    if (uti != NULL) {
        CFStringRef mime = UTTypeCopyPreferredTagWithClass(uti, kUTTagClassMIMEType);
        CFRelease(uti);
        if (mime != NULL) {
            NSString *type = [NSString stringWithString:(__bridge NSString *)mime];
            CFRelease(mime);
            return type;
        }
    }

    return @"image/jpeg";
}

@end
