//
//  FHResults.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/14/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"


@interface FHRemoteArray ()
@property (nonatomic, copy             ) NSMutableOrderedSet *backingStore;
@property (nonatomic, strong           ) FetchBlock          fetchBlock;
@property (nonatomic, assign, readwrite) NSInteger           totalRows;
@property (nonatomic, assign           ) BOOL                fresh;
@end


@implementation FHRemoteArray

- (id)initWithFetchBlock:(FetchBlock)fetchBlock
{
    assert(fetchBlock != nil);
    self = [super init];
    if (self) {
        _totalRows      = 0;
        _backingStore   = [NSMutableOrderedSet orderedSet];
        _perPage        = -1;
        _page           = 1;
        _fetchBlock     = fetchBlock;
        _fresh          = YES;
    }
    return self;
}

+ (FHRemoteArray *)remoteArrayWithFetchBlock:(FetchBlock)fetchBlock
{
    return [[FHRemoteArray alloc] initWithFetchBlock:fetchBlock];
}

+ (FHRemoteArray *)remoteArrayWithPath:(NSString *)path
                           identifiers:(NSArray *)identifiers
                                params:(NSDictionary *)params
                     createObjectBlock:(id (^)(NSDictionary *dict))createObjectBlock
{
    FHRemoteArray *remoteArray = [FHRemoteArray remoteArrayWithFetchBlock:^MTPocketRequest *(FHRemoteArray *remoteArray, NSInteger page, NSInteger perPage) {
        MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                      route:path
                                                                identifiers:identifiers
                                                                     method:MTPocketMethodGET
                                                                       page:page
                                                                    perPage:perPage
                                                                     params:params
                                                                       body:nil];

        [request addSuccess:^(MTPocketResponse *response) {
            if ([response.body isKindOfClass:[NSArray class]]) {
                if ([response.body lastObject][@"total_rows"]) {
                    remoteArray.totalRows = [[response.body lastObject][@"total_rows"] integerValue];
                }
                for (NSDictionary *dict in response.body) {
                    FHObject *object = createObjectBlock(dict);
                    [object populateWithDictionary:dict];
                    [remoteArray.backingStore addObject:object];
                }
            }
        }];

        return request;
    }];

    return remoteArray;
}



- (MTPocketRequest *)next
{
    if (!_fresh && [self count] == _totalRows) return nil;
    _fresh = NO;
    return _fetchBlock(self, _page++, _perPage);
}

- (void)empty
{
    [_backingStore removeAllObjects];
}

- (void)reset
{
    [self empty];
    _totalRows  = 0;
    _fresh      = YES;
    _page       = 1;
}





#pragma mark NSArray

- (NSUInteger)count
{
    return [_backingStore count];
}

- (id)objectAtIndex:(NSUInteger)index
{
    return _backingStore[index];
}

- (id)objectAtIndexedSubscript:(NSUInteger)idx
{
    return _backingStore[idx];
}

- (NSUInteger)indexOfObject:(id)object
{
    return [_backingStore indexOfObject:object];
}


#pragma mark NSMutableArray

- (void)insertObject:(id)anObject atIndex:(NSUInteger)index
{
    [_backingStore insertObject:anObject atIndex:index];
}

- (void)removeObjectAtIndex:(NSUInteger)index
{
    [_backingStore removeObjectAtIndex:index];
}

- (void)removeObject:(id)object
{
    [_backingStore removeObject:object];
}

- (void)addObject:(id)anObject
{
    [_backingStore addObject:anObject];
}

- (void)removeLastObject
{
    [_backingStore removeObjectAtIndex:[_backingStore count] - 1];
}

- (void)replaceObjectAtIndex:(NSUInteger)index withObject:(id)anObject
{
    _backingStore[index] = anObject;
}


@end
