//
//  FHTag.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/26/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"



@interface FHTag ()
@property (nonatomic, assign, readwrite) FHCompany *company;
@end



@implementation FHTag



+ (FHTag *)tagForCompany:(FHCompany *)company withLabel:(NSString *)label
{
    FHTag *tag  = [FHTag tagWithIdentifier:nil company:company];
    tag.label   = label;
    return tag;
}

- (MTPocketRequest *)save
{
    MTPocketRequest *request    = nil;
    NSDictionary *body          = @{ @"tag" : [self exportDictionary] };

    if (!self.identifier) {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"companies/:company_id/tags"
                                               identifiers:@[self.company.identifier]
                                                    method:MTPocketMethodPOST
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
        [request addSuccess:^(MTPocketResponse *response) {
            [self populateWithDictionary:response.body];
            [(NSMutableOrderedSet *)_company.tags addObject:self];
            self.isFetched = YES;
        }];
    }
    else {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"tags/:id"
                                               identifiers:@[self.identifier]
                                                    method:MTPocketMethodPUT
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
    }

    return request;
}

- (MTPocketRequest *)destroy
{
    assert(self.identifier != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"tags/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_company.tags removeObject:self];
    }];

    return request;
}





#pragma mark - Protected

+ (FHTag *)tagWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company
{
    FHTag *tag  = (FHTag *)[self objectWithIdentifier:identifier];
    tag.company = company;
    return tag;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _label = [FHObject importString:dictionary[@"label"]];
    [super populateWithDictionary:dictionary];
}

- (NSDictionary *)exportDictionary
{
    [self exportString:_label as:@"label"];
    return [super exportDictionary];
}

@end
