//
//  FHTwitterAccount.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/26/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"


@interface FHTwitterAccount ()
@property (nonatomic, assign, readwrite) FHCompany *company;
@end




@implementation FHTwitterAccount


+ (NSURL *)OAuthURLForCompany:(FHCompany *)company withCallback:(NSString *)callback
{
    assert(company != nil);
    assert(callback != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestWithTemplate:@"api"
                                                                       path:@"companies/:company_id/oauth_twitter"
                                                                identifiers:@[company.identifier]
                                                                     method:MTPocketMethodGET
                                                                       body:nil
                                                                     params:@{ @"url_token" : [FHClient sharedClient].URLToken, @"callback_url" : callback }];
    return [request resolvedURL];
}

- (MTPocketRequest *)destroy
{
    assert(self.identifier != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"twitter_accounts/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [(NSMutableOrderedSet *)_company.twitterAccounts removeObject:self];
    }];

    return request;
}


#pragma mark - Protected

+ (FHTwitterAccount *)twitterAccountWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company
{
    FHTwitterAccount *twitterAccount = (FHTwitterAccount *)[self objectWithIdentifier:identifier];
    twitterAccount.company = company;
    return twitterAccount;
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _screenName     = [FHObject importString:dictionary[@"screen_name"]];
    _twitterUserId  = [FHObject importNumber:dictionary[@"twitter_user_id"]];
    _imageURL       = [FHObject importURL:dictionary[@"image_url"]];
    
    [super populateWithDictionary:dictionary];
}


@end
