//
//  private.h
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/11/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import <NSDate+MTDates.h>
#import <MTPocket.h>
#import "FHClient.h"
#import "FHAgent.h"
#import "FHCompany.h"
#import "FHCustomer.h"
#import "FHInteraction.h"
#import "FHTwitterInteraction.h"
#import "FHFacebookInteraction.h"
#import "FHEmailInteraction.h"
#import "FHCustomerAccount.h"
#import "FHNotification.h"
#import "FHTwitterAccount.h"
#import "FHEmailAccount.h"
#import "FHFacebookAccount.h"
#import "FHFacebookPage.h"
#import "FHAgentInvite.h"
#import "FHTag.h"
#import "FHCannedResponse.h"
#import "FHNote.h"
#import "FHAttachment.h"
#import "FHOutgoingAttachment.h"
#import "FHRemoteArray.h"
#import "FHCreditCard.h"

// swaps NSNull with nil
#define NILL(a) ([a isKindOfClass:[NSNull class]] ? nil : a)

// swaps nil with NSNull
#define NUL(a) (a ? a : [NSNull null])

// Only overwrites left if left is nil
#define FAVOR_LEFT(a,b) (a = !a ? b : a)

// Only overwrites left if right is NOT nil
#define FAVOR_RIGHT(a,b) (a = b ? b : a)







#pragma mark - FHObject

@interface FHObject ()

@property (nonatomic, copy,   readwrite) NSNumber *identifier;
@property (nonatomic, assign, readwrite) BOOL     isFetched;

+ (FHObject *)objectWithIdentifier:(NSNumber *)identifier;
- (BOOL)isAssociated:(NSOrderedSet *)associated;

#pragma mark - Import

- (void)populateWithDictionary:(NSDictionary *)dictionary;

- (void)populateAssociatedObjects:(NSOrderedSet *__strong *)associatedObjects
                 withDictionaries:(NSArray *)dictionaries
                createObjectBlock:(id (^)(NSDictionary *dict))createObjectBlock;

- (void)addToAssociatedObjects:(NSOrderedSet *)associatedObjects
                withDictionary:(NSDictionary *)dictionary
             createObjectBlock:(id (^)(NSDictionary *dict))createObjectBlock;

- (void)populateAssociatedObject:(FHObject *__strong *)associatedObject
                  withDictionary:(NSDictionary *)dictionary
               createObjectBlock:(id (^)(NSDictionary *dict))createObjectBlock;

- (void)populateAssociatedObject:(FHObject *__strong *)associatedObject
                  withIdentifier:(NSNumber *)identifier
               createObjectBlock:(id (^)(NSNumber *identifier))createObjectBlock;

+ (int)importEnum:(NSString *)string usingMap:(NSDictionary *)map;
+ (NSString *)importString:(NSString *)string;
+ (NSNumber *)importNumber:(NSString *)string;
+ (BOOL)importBool:(NSNumber *)value;
+ (NSInteger)importInteger:(NSNumber *)number;
+ (CGFloat)importFloat:(NSNumber *)number;
+ (NSDate *)importDate:(NSString *)string;
+ (NSURL *)importURL:(NSString *)string;

#pragma mark - Export

- (void)exportEnum:(int)enumValue               as:(NSString *)key      usingMap:(NSDictionary *)map;
- (void)exportString:(NSString *)stringValue    as:(NSString *)key;
- (void)exportNumber:(NSNumber *)numberValue    as:(NSString *)key;
- (void)exportBool:(BOOL)boolValue              as:(NSString *)key;
- (void)exportInteger:(NSInteger)integerValue   as:(NSString *)key;
- (void)exportDate:(NSDate *)dateValue          as:(NSString *)key;
- (void)exportURL:(NSURL *)URLValue             as:(NSString *)key;

- (NSDictionary *)exportDictionary;

@end






#pragma mark - FHClient

@interface FHClient ()
@property (nonatomic, strong) NSString *token;
@property (nonatomic, strong) NSString *URLToken;
@property (nonatomic, strong) NSString *billingToken;
@end


typedef MTPocketRequest *(^FetchBlock)(FHRemoteArray *remoteArray, NSInteger page, NSInteger perPage);
@interface FHRemoteArray ()
+ (FHRemoteArray *)remoteArrayWithFetchBlock:(FetchBlock)fetchBlock;
+ (FHRemoteArray *)remoteArrayWithPath:(NSString *)path
                           identifiers:(NSArray *)identifiers
                                params:(NSDictionary *)params
                     createObjectBlock:(id (^)(NSDictionary *dict))createObjectBlock;
@end





#pragma mark - FHInteraction

@interface FHInteraction ()
@property (nonatomic, assign, readwrite) FHCustomer *customer;
@property (nonatomic, strong, readwrite) FHAgent    *agent;
+ (FHInteraction *)interactionWithDictionary:(NSDictionary *)dictionary customer:(FHCustomer *)customer;
+ (NSDictionary *)channelEnumMap;
@end

@interface FHTwitterInteraction ()
+ (FHTwitterInteraction *)twitterInteractionWithIdentifier:(NSNumber *)identifier;
@end

@interface FHFacebookInteraction ()
+ (FHFacebookInteraction *)facebookInteractionWithIdentifier:(NSNumber *)identifier;
@end

@interface FHEmailInteraction ()
@property (nonatomic, strong) NSString *emailSubject;
+ (FHEmailInteraction *)emailInteractionWithIdentifier:(NSNumber *)identifier;
@end








@interface FHCompany ()
+ (FHCompany *)companyWithIdentifier:(NSNumber *)identifier agent:(FHAgent *)agent;
@end

@interface FHCustomer ()
+ (FHCustomer *)customerWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company;
@end

@interface FHCustomerAccount ()
+ (FHCustomerAccount *)customerAccountWithIdentifier:(NSNumber *)identifier customer:(FHCustomer *)customer;
@end

@interface FHNotification ()
+ (FHNotification *)notificationWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company;
@end

@interface FHTwitterAccount ()
+ (FHTwitterAccount *)twitterAccountWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company;
@end

@interface FHEmailAccount ()
+ (FHEmailAccount *)emailAccountWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company;
@end

@interface FHFacebookAccount ()
+ (FHFacebookAccount *)facebookAccountWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company;
@end

@interface FHFacebookPage ()
+ (FHFacebookPage *)facebookPageWithIdentifier:(NSNumber *)identifier facebookAccount:(FHFacebookAccount *)facebookAccount;
@end

@interface FHAgentInvite ()
+ (FHAgentInvite *)agentInviteWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company;
@end

@interface FHTag ()
+ (FHTag *)tagWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company;
@end

@interface FHCannedResponse ()
+ (FHCannedResponse *)cannedResponseWithIdentifier:(NSNumber *)identifier company:(FHCompany *)company;
@end

@interface FHNote ()
+ (FHNote *)noteWithIdentifier:(NSNumber *)identifier interaction:(FHInteraction *)interaction;
@end

@interface FHAttachment ()
+ (FHAttachment *)attachmentWithIdentifier:(NSNumber *)identifier emailInteraction:(FHEmailInteraction *)emailInteraction;
@end

@interface FHCreditCard ()
+ (FHCreditCard *)creditCardWithCompany:(FHCompany *)company;
@end





